<?php
/**
 * Console Library
 *
 * LICENSE
 *
 * "THE BEER-WARE LICENSE" (Revision 42):
 * "Sven Strittmatter" <ausserirdisch@sven-space.de> wrote this file.
 * As long as you retain this notice you can do whatever you want with
 * this stuff. If we meet some day, and you think this stuff is worth it,
 * you can buy me a beer in return.
 *
 * $Author: sxs $
 * $Revision: 161 $
 * 
 * @category    Console
 * @package     Console_Application
 * @copyright   Copyright (c) 2009 Sven Strittmatter
 */

/**
 * Console_Loader.
 *
 * @see Console_Loader
 */
require_once('Console/Loader.php');
/**
 * Console_Exception.
 *
 * @see Console_Exception
 */
require_once('Console/Exception.php');

/**
 * Class for running a Console_Application_Abstract-Script.
 *
 * @final
 * @category   Console
 * @package    Console_Application
 * @uses       Console_Loader
 * @uses       Console_Exception
 * @copyright  Copyright (c) 2009 Sven Strittmatter
 */
final class Console_Application {
    /**
     * Loads a class named ${applicationName}Application in the direcotry
     * ${directory} and executes it.
     *
     * @static
     * @access public
     * @param string $applicationName
     * @param string $direcotry Optional;
     * @param int $maxExecutionTime Optional;
     * @param string $memoryLimit Optional;
     * @return Console_Application_Interface
     * @throws Console_Exception If file is not readable
     */
    public static function execute($applicationName, $direcotry = '.', $maxExecutionTime = 60, $memoryLimit = '128M') {
        $className  = $applicationName . 'Application';

        if (!class_exists($className, false)) {
            $fileName = $direcotry . DIRECTORY_SEPARATOR . $className . '.php';
            
            if (Console_Loader::isReadable($fileName)) {
                require_once($fileName);
            } else {
                throw new Console_Exception('Cant require console file: ' . $fileName);
            }
        }

        $scriptName  = strtolower($applicationName) . '.sh';
        $application = new $className($scriptName, $maxExecutionTime, $memoryLimit);

        if (!$application instanceof Console_Application_Interface) {
            throw new Console_Exception($className . ' must implement the interface Console_Application_Interface!');
        }

        $application->execute();

        return $application;
    }
}
