<?php
/**
 * Console Library
 *
 * LICENSE
 *
 * "THE BEER-WARE LICENSE" (Revision 42):
 * "Sven Strittmatter" <ausserirdisch@sven-space.de> wrote this file.
 * As long as you retain this notice you can do whatever you want with
 * this stuff. If we meet some day, and you think this stuff is worth it,
 * you can buy me a beer in return.
 *
 * $Author: sxs $
 * $Revision: 161 $
 *
 * @category    Console
 * @package     Console_Application
 * @copyright   Copyright (c) 2009 Sven Strittmatter
 */
 
/**
 * Console_Exception.
 *
 * @see Console_Exception
 */
require_once('Console/Exception.php');

/**
 * Class for running a Console_Application_Abstract-Script.
 *
 * @final
 * @category   Console
 * @package    Console_Application
 * @uses       Console_Exception
 * @copyright  Copyright (c) 2009 Sven Strittmatter
 */
final class Console_Loader {
    /**
     * Loads a class depending on the class name.
     * All _ in the classname are replaced with DIRECTORY_SEPARATOR.
     * 
     * @static
     * @access public
     * @param string $name
     * @throws Console_Exception If the generated filename is not valid
     * @throws Console_Exception If the file is not readable
     * @throws Console_Exception If the file doesnt contain the class/interface
     */
    public static function loadClass($name) {
        if (class_exists($name, false) || interface_exists($name, false)) {
            return;
        }

        $file = str_replace('_', DIRECTORY_SEPARATOR, $name) . '.php';

        if (!self::isFileNameValid($file)) {
            $message = 'Illegal character in filename: ' . $file;
            throw new Console_Exception($message);
        }

        if (!self::isReadable($file)) {
            $message = "File: $file is not readable!";
            throw new Console_Exception($message);
        }

        require_once($file);

        if (!class_exists($name, false) && !interface_exists($name, false)) {
            $message  = "Class or interface '$name' ";
            $message .= "was not found in the file '$file'";
            throw new Console_Exception($message);
        }
    }

    /**
     * Checks if a file is readable.
     *
     * @static
     * @access public
     * @param  string $filename
     * @return bool
     */
    public static function isReadable($filename) {
        if (!$fh = @fopen($filename, 'r', true)) {
            return false;
        }

        @fclose($fh);

        return true;
    }

    /**
     * Checks if a filename is valid.
     *
     * @static
     * @access public
     * @param  string $filename
     * @return bool
     */
    public static function isFileNameValid($filename) {
        if (0 === preg_match('/[^a-z0-9\\/\\\\_.:-]/i', $filename)) {
            return true;
        } else {
            return false;
        }
    }
}
