<?php
/**
 * Util
 *
 * LICENSE
 *
 * "THE BEER-WARE LICENSE" (Revision 42):
 * "Sven Strittmatter" <ausserirdisch@sven-space.de> wrote this file.
 * As long as you retain this notice you can do whatever you want with
 * this stuff. If we meet some day, and you think this stuff is worth it,
 * you can buy me a beer in return.
 *
 * @category    Util
 * @package     Benchmark
 * @copyright   Copyright (c) 2007 Sven Strittmatter
 */

/**
 * Inspiriert von der Benchmark-Klasse von Kohana (http://kohanaphp.com/).
 *
 * @category    Util
 * @package     Benchmark
 * @author	"Sven Strittmatter" <ausserirdisch@sven-space.de>
 * @copyright   Copyright (c) 2007 Sven Strittmatter
 */
class Util_Benchmark_Individual {
    /**
     * Name of the benchmark
     *
     * @access private
     * @var string
     */
    private $name;
    /**
     * Array of Util_Benchmark_Mark objects.
     * Each time you start an individual benchmark
     * one mark object will be added.
     * @var array
     */
    private $marks;
    /**
     * Flag signals if the individual benchmark is still running.
     *
     * @access private
     * @var bool
     */
    private $isRunning;

    /**
     * Does nothing special.
     *
     * @param string $name
     */
    public function  __construct($name) {
        $this->name  = $name;
        $this->reset();
    }

    /**
     * Returns the name to identify the benchmark individual.
     *
     * @access public
     * @return string
     */
    public function getName() {
        return $this->name;
    }

    /**
     * Starts the individual benchmark. You can perfrom the start
     * as often you want to restart the benchmarking after stopping it.
     * Until you perform a reset the time and memory usage will be accumualted.
     *
     * @access public
     */
    public function start() {
        if (!$this->isRunning) {
            $this->marks[]   = new Util_Benchmark_Mark();
            $this->isRunning = true;
        }
    }

    /**
     * Stops the individual benchmark.
     *
     * @access public
     */
    public function stop() {
        if ($this->hasMarks() && $this->isRunning) {
            end($this->marks)->stop();
            $this->isRunning = false;
        }
    }

    /**
     * Returns if there are single mark objects.
     *
     * @access private
     * @return bool
     */
    private function hasMarks() {
        return count($this->marks) > 0;
    }

    /**
     * Resets the individual benchmark.
     *
     * @access public
     */
    public function reset() {
        $this->marks     = array();
        $this->isRunning = false;
    }

    /**
     * Returns the accumulated time.
     *
     * @access public
     * @return float
     */
    public function getTime() {
        if (!$this->hasMarks()) {
            return 0;
        }

        $time = 0;

        foreach ($this->marks as $$mark) {
            $time += $mark->getTime();
        }

        return $time;
    }

    /**
     * Returns the accumulated memory usage.
     *
     * @access public
     * @return int
     */
    public function getMemUsage() {
        if (!$this->hasMarks()) {
            return 0;
        }

        $memory = 0;

        foreach ($this->marks as $$mark) {
            $memory += $mark->getMemUsage();
        }

        return $memory;
    }
}