<?php
/**
 *
 * Util
 *
 * LICENSE
 *
 * "THE BEER-WARE LICENSE" (Revision 42):
 * "Sven Strittmatter" <ausserirdisch@sven-space.de> wrote this file.
 * As long as you retain this notice you can do whatever you want with
 * this stuff. If we meet some day, and you think this stuff is worth it,
 * you can buy me a beer in return.
 *
 * @category    Util
 * @package     Benchmark
 * @copyright   Copyright (c) 2007 Sven Strittmatter
 */

/**
 * Inspiriert von der Benchmark-Klasse von Kohana (http://kohanaphp.com/).
 *
 * @category    Util
 * @package     Benchmark
 * @author	"Sven Strittmatter" <ausserirdisch@sven-space.de>
 * @copyright   Copyright (c) 2007 Sven Strittmatter
 */
class Util_Benchmark_Mark {
    /**
     * The start time.
     *
     * @access private
     * @var float
     */
    private $startTime;
    /**
     * The stop time.
     *
     * @access private
     * @var float
     */
    private $stopTime;
    /**
     * Amount of used memory on start time.
     *
     * @access private
     * @var int
     */
    private $startMemUsage;
    /**
     * Amount of used memory on stop time.
     *
     * @access private
     * @var int
     */
    private $stopMemUsage;
    /**
     * Flag signals if the mark is still running.
     *
     * @access private
     * @var bool
     */
    private $isRunning;

    /**
     * The mark "starts" on construction time. This means that start time
     * and start memory usage is set on constuction time. No explicit starting
     * neccessary.
     *
     * @access public
     */
    public function  __construct() {
        $this->startTime     = microtime(true);
        $this->startMemUsage = $this->getMemoryUsage();
        $this->isRunning     = true;
    }

    /**
     * Returns true until the mark is stopped.
     *
     * @access public
     * @return bool
     */
    public function isRunning() {
        return $this->isRunning;
    }

    /**
     * Sets memory usage and time on stop.
     *
     * @access public
     */
    public function stop() {
        if ($this->isRunning()) {
            $this->stopTime     = microtime(true);
            $this->stopMemUsage = $this->getMemoryUsage();
            $this->isRunning    = false;
        }
    }

    /**
     * Returns the time this mark runned.
     * Stops the mark implicit if its allready running.
     *
     * @access public
     * @return float
     */
    public function getTime() {
        if ($this->isRunning()) {
            $this->stop();
        }

        return $this->stopTime - $this->startTime;
    }

    /**
     * Returns the memory used this mark runned.
     * Stops the mark implicit if its allready running.
     *
     * @access public
     * @return int
     */
    public function getMemUsage() {
        if ($this->isRunning()) {
            $this->stop();
        }

        return $this->stopMemUsage - $this->startMemUsage;
    }

    /**
     * Checks if the php function memory_get_usage() exists an returns
     * its return value.
     *
     * @staticvar bool $functionExists
     * @param bool $realUsage
     * @return int
     */
    private function getMemoryUsage($realUsage = false) {
        /**
         * I make this static so the check if the function
         * exits is only made one time peer request.
         */
        static $functionExists;

        if (null === $functionExists) {
            // Test if memory usage can be seen
            $functionExists = function_exists('memory_get_usage');
        }

        return $functionExists ? memory_get_usage($realUsage) : 0;
    }
}