<?php
/**
 * Util
 *
 * LICENSE
 *
 * "THE BEER-WARE LICENSE" (Revision 42):
 * "Sven Strittmatter" <ausserirdisch@sven-space.de> wrote this file.
 * As long as you retain this notice you can do whatever you want with
 * this stuff. If we meet some day, and you think this stuff is worth it,
 * you can buy me a beer in return.
 *
 * @category    Util
 * @package     PasswordGenerator
 * @copyright   Copyright (c) 2007 Sven Strittmatter
 */

/**
 * TODO
 *
 * TODO
 *
 * @category    Util
 * @package     String
 * @author	"Sven Strittmatter" <ausserirdisch@sven-space.de>
 * @copyright   Copyright (c) 2007 Sven Strittmatter
 */
class Util_String implements IteratorAggregate, ArrayAccess, Countable {
    protected $str;

    // static methods
    /**
     * chr — Gibt ein einzelnes Zeichen zurück
     */
    public static function getCharacter($ascii) {
        return new Util_String(chr($ascii));
    }

    // interceptors
    public function __construct($str = '') {
        $this->str = $str;
    }

    public function __clone() {
        return new Util_String($this->str);
    }

    public function __toString() {
        return $this->str;
    }

    // handy methods
    public function isEqual(Util_String $s) {
        return (self::IS_EQUAL === $this->compare($s, false));
    }

    public function append(Util_String $str) {
        return new Util_String($this->str.$str);
    }

    public function prepend(Util_String $str) {
        return new Util_String($str.$this->str);
    }

    public function wrap(Util_String $str) {
        return new Util_String($str.$this->str.$str);
    }

    // implmenting IteratorAggregate:
    public function getIterator() {
        return new ArrayIterator($this);
    }

    // implmenting ArrayAccess:
    public function offsetExists($offset) {
        if (0 <= $offset && $this->length() > $offset) {
            return true;
        }

        return false;
    }

    public function offsetGet($offset) {
        return $this->str[$offset];
    }

    public function offsetSet($offset, $value) {
        throw new BadMethodCallException('Util_String is imutable!');
    }

    public function offsetUnset($offset) {
        throw new BadMethodCallException('Util_String is imutable!');
    }

    // implmenting Countable:
    public function count() {
        return $this->length();
    }

    // standard php API
    public function length() {
        return strlen($this->str);
    }
    /**
     * addcslashes — Stellt bestimmten Zeichen eines Strings ein "\" voran (wie in C)
     */
    public function addCSlashes($characterList) {
        return new Util_String(addcslashes($this->str, $characterList));
    }

    /**
     * stripcslashes — Entfernt Quotes aus mit addcslashes behandelten Strings
     */
    public function stripCSlashes() {
        return new Util_String(stripcslashes($this->str));
    }

    /**
     * addslashes — Stellt bestimmten Zeichen eines Strings ein "\" voran
     */
    public function addSlashes() {
        return new Util_String(addslashs($this->str));
    }

    /**
     * stripslashes — Entfernt aus einem gequoteten String alle Quotes
     */
    public function stripSlashes() {
        return new Util_String(stripslashes($this->str));
    }

    /**
     * bin2hex — Wandelt Binär-Daten in ihre hexadezimale Entsprechung um
     */
    public function toHex() {
        return new Util_String(bin2hex($this->str));
    }

    /**
     * chunk_split — Zerlegt einen String in Teile gleicher Länge
     */
    public function chunkSplit($chunkLength, $chunkEnd = "\r\n") {
        return new Util_String(chunk_split($this->str, $chunkLength, $chunkEnd));
    }

    const CYR_CHARSET_KOI8_R       = 'k';
    const CYR_CHARSET_WINDOWS_1251 = 'w';
    const CYR_CHARSET_ISO8859_5    = 'i';
    const CYR_CHARSET_X_CP866_A    = 'a';
    const CYR_CHARSET_X_CP866_D    = 'd';
    const CYR_CHARSET_X_MAC        = 'm';
    /**
     * Convert from one Cyrillic character set to another
     *
     *  Supported characters are:
     * k - koi8-r
     * w - windows-1251
     * i - iso8859-5
     * a - x-cp866
     * d - x-cp866
     * m - x-mac-cyrillic
     *
     * @param $from The source Cyrillic character set, as a single character.
     * @param $to The target Cyrillic character set, as a single character.
     */
    public function convertCyrillic($fromCharacterSet, $toCharacterSet) {
        return new Util_String(convert_cyr_string($this->str, $fromCharacterSet, $toCharacterSet));
    }

    /**
     * convert_uuencode — UU-kodiert eine Zeichenkette
     */
    public function uuencode() {
        return new Util_String(convert_uuencode($this->str));
    }

    /**
     * convert_uudecode — Dekodiert eine uukodierte Zeichenkette
     */
    public function uudecode() {
        return new Util_String(convert_uudecode($this->str));
    }


    /**
     * count_chars — Gibt Informationen über die in einem String enthaltenen Zeichen zurück
     *
     * 0 - an array with the byte-value as key and the frequency of every byte as value.
     * 1 - same as 0 but only byte-values with a frequency greater than zero are listed.
     * 2 - same as 0 but only byte-values with a frequency equal to zero are listed.
     * 3 - a string containing all unique characters is returned.
     * 4 - a string containing all not used characters is returned.
     */
    public function countCharacters($mode = 0) {
        return count_chars($this->str, $mode);
    }

    /**
     * explode — Teilt einen String anhand einer Zeichenkette
     */
    public function explode($delimiter) {
        return explode($delimiter, $this->str);
    }

    /**
     * implode — Verbindet Array-Elemente zu einem String
     */
    public static function implode($delimiter, array $pieces) {
        return new Util_String(implode($delimiter, $pieces));
    }

    /**
     * lcfirst — Wandelt den ersten Buchstaben eines Strings in einen Kleinbuchstaben um
     *
     * @return Util_String
     */
    public function lcFirst() {
        static $functionExists = null;

        if (null === $functionExists) {
            $functionExists = function_exists('lcfirst');
        }

        if ($functionExists) {
            $newString = lcfirst($this->str);
        } else {
            $newString = $this->str;
            $newString[0] = strtolower($newString[0]);
        }

        return new Util_String($newString);
    }

    /**
     * ucfirst — Verwandelt das erste Zeichen eines Strings in einen Großbuchstaben
     *
     * @return Util_String
     */
    public function ucFirst() {
        return new Util_String(ucfirst($this->str));
    }

    /**
     * strtolower — Setzt einen String in Kleinbuchstaben um
     *
     * @return Util_String
     */
    public function toLowerCase() {
        return new Util_String(strtolower($this->str));
    }

    /**
     * strtoupper — Wandelt alle Zeichen eines Strings in Großbuchstaben um
     *
     * @return Util_String
     */
    public function toUpperCase() {
        return new Util_String(strtoupper($this->str));
    }

    const TRIM_LEFT = 0;
    const TRIM_RIGHT = 1;
    const TRIM_BOTH = 2;

    /**
     * Stripping whitespaces from the beginning and end of a string.
     *
     * @param $mode
     * @return unknown_type
     */
    public function trim($mode = Util_String::TRIM_BOTH) {
        if (self::TRIM_LEFT === $mode) {
            return new Util_String(ltrim($this->str));
        } else if (self::TRIM_BOTH === $mode) {
            return new Util_String(trim($this->str));
        } else if (self::TRIM_RIGHT === $mode) {
            return new Util_String(rtrim($this->str));
        } else {
            $msg = "@todo";
            throw new InvalidArgumentException($msg);
        }
    }

    /**
     * nl2br — Fügt vor allen Zeilenumbrüchen eines Strings HTML-Zeilenumbrüche ein
     *
     * @return unknown_type
     */
    public function nlToBr() {
        return new Util_String(nl2br($this->str));
    }

    /**
     * str_ireplace — Groß- und kleinschreibungsunabhängige Version von str_replace
     */
    public function replace($search, $replacement, $caseInsensitive = true, $count = null) {
        if ($caseInsensitive) {
            return new Util_String(str_replace($search, $replacemnt, $this->str, $count));
        } else {
            return new Util_String(str_ireplace($search, $replacemnt, $this->str, $count));
        }
    }

    const IS_LESS    = -1;
    const IS_EQUAL   = 0;
    const IS_GREATER = 1;

    /**
     * strcasecmp — Vergleich von Zeichenketten ohne Unterscheidung der Groß- und Kleinschreibung (Binary safe)
     * strcmp — Vergleich zweier Strings (Binary safe)
     */
    public function compare(Util_String $comparingString, $caseInsensitive = true) {
        if ($caseInsensitive) {
            return strcasecmp($this->str, $comparingString->__toString());
        } else {
            return strcmp($this->str, $comparingString->__toString());
        }
    }

    const PAD_RIGHT = STR_PAD_RIGHT;
    const PAD_LEFT  = STR_PAD_LEFT;
    const PAD_BOTH  = STR_PAD_BOTH;

    /**
     * str_pad — Erweitert einen String unter Verwendung eines anderen Strings auf eine bestimmte Länge
     */
    public function pad($length, $string = '' , $type = Util_String::PAD_RIGHT) {
        return new Util_String(str_pad($this->str, $length, $string, $type));
    }

    /**
     * str_repeat — Wiederholt einen String
     */
    public function repeat($times) {
        return new Util_String(str_repeat($this->str, $times));
    }

    /**
     * str_shuffle — Mischt einen String nach dem Zufallsprinzip
     */
    public function shuffle() {
        return new Util_String(str_shuffle($this->str));
    }

    /**
     * str_split — Konvertiert einen String in ein Array
     *
     * @return array
     */
    public function split($length = 1) {
        return str_split($this->str, $length);
    }

    const WC_FORMAT_NUMBER = 0;
    const WC_FORMAT_ARRAY  = 1;
    const WC_FORMAT_ASSOC  = 2;

    /**
     * str_word_count — Gibt Informationen über in einem String verwendete Worte zurück
     */
    public function wordCount($format = Util_String::WC_FORMAT_NUMBER, $characters = '') {
        return str_word_count($this->string, $format, $characters);
    }

    /**
     * strip_tags — Entfernt HTML- und PHP-Tags aus einem String
     */
    public function stripTags($allowableTags = '') {
        return new Util_String(strip_tags($this->str, $allowableTags));
    }

    /**
     *  strrev — Kehrt einen String um
     */
    public function reverse() {
        return new Util_String(strrev($this->str));
    }

    /**
     * strpos — Sucht das erste Vorkommen des Suchstrings
     * stripos — Findet das erste Vorkommen eines Strings, unabhängig von Groß- und Kleinschreibung
     *
     * @return unknown_type
     */
    public function position($needle, $offset = 0, $caseInsensitive = true) {
        if ($caseInsensitive) {
            return strpos($this->str, $needle, $offset);
        } else {
            return stripos($this->str, $needle, $offset);
        }
    }

    /**
     * strripos — Findet das letzte Vorkommen der gesuchten Zeichenkette in einem String, unabhängig von Groß- und Kleinschreibung
     * strrpos — Findet das letzte Vorkommen eines Zeichens innerhalb einer Zeichenkette
     *
     * @param $caseInsensitive
     * @return unknown_type
     */
    public function lastPosition($needle, $offset = 0, $caseInsensitive = true) {
        if ($caseInsensitive) {
            return strrpos($this->str, $needle, $offset);
        } else {
            return strripos($this->str, $needle, $offset);
        }
    }

     /*
    * strspn — Ermittelt die Länge der übereinstimmenden Zeichen
    * strstr — Findet das erste Vorkommen eines Strings
    * strtok — Zerlegt einen String
    * strtr — Tauscht bestimmte Zeichen aus
    * substr_compare — Binärdaten-sicherer Vergleich zweier Strings, beginnend an einer bestimmten Position und endend nach einer festgelegten Länge
    * substr_count — Ermittelt, wie oft eine Zeichenkette in einem String vorkommt
    * substr_replace — Ersetzt Text innerhalb einer Zeichenkette
    * substr — Gibt einen Teil eines Strings zurück
    * ucwords — Wandelt jeden ersten Buchstaben eines Wortes innerhalb eines Strings in einen Großbuchstaben
    * vfprintf — Schreibt einen formatierten String in einen Stream
    * vprintf — Gibt einen formatierten String zurück
    * vsprintf — Gibt einen formatierten String zurück
    * wordwrap — Bricht einen String nach einer bestimmten Anzahl Zeichen um
    * stristr — Wie strstr, aber ohne Rücksicht auf Groß- bzw. Kleinschreibung
    * strnatcasecmp — String-Vergleich "natürlicher Ordnung" ohne Unterscheidung der Schreibweise
    * strnatcmp — String-Vergleich unter Verwendung einer "natürlichen Ordnung"
    * strncasecmp — Binärdaten-sicherer und groß- und kleinschreibungs-unabhängiger Stringvergleich der ersten n Zeichen
    * strncmp — String-Vergleich der ersten n Zeichen (Binary safe)
    * strpbrk — Durchsucht einen String nach einem Zeichen aus einer Gruppe von Zeichen
    * strrchr — Sucht das letzte Vorkommen eines Zeichens in einem String
    * levenshtein — Berechnet die Levenshtein-Distanz zwischen zwei Strings
    * localeconv — Ermittelt die Formatierungsinformationen für Zahlen
    * metaphone — Berechnet den Metaphone-Schlüssel eines Strings
    * money_format — Formats a number as a currency string
    * nl_langinfo — Query-Language und Locale Information
    * number_format — Formatiert eine Zahl mit Tausender-Gruppierung
    * ord — Gibt den ASCII-Wert eines Zeichens zurück
    * parse_str — Überträgt einen String in Variable
    * printf — Gibt einen formatierten String aus
    * quoted_printable_decode — Konvertiert einen "quoted-printable"-String in einen 8-Bit-String
    * quoted_printable_encode — Convert a 8 bit string to a quoted-printable string
    * quotemeta — Quoten von Meta-Zeichen
    * setlocale — Setzt Locale Informationen
    * similar_text — Berechnet die Ähnlichkeit zweier Zeichenketten
    * soundex — Berechnet die Laut-Ähnlichkeit eines Strings
    * sprintf — Gibt einen formatierten String zurück
    * sscanf — Überträgt einen String in ein angegebenes Format
    * str_getcsv — Parst einen CSV-String in ein Array
    * fprintf — Schreibt einen formatierten String in einen Stream
    * get_html_translation_table — Gibt die Umwandlungs-Tabelle zurück, die von htmlspecialchars und htmlentities verwendet wird
    * hebrev — Konvertiert logischen hebräischen Text in sichtbaren Text
    * hebrevc — Konvertiert (natürlichen) hebräischen Text in sichtbaren Text inkl. Anpassung von Zeilenumbrüchen
    * html_entity_decode — Konvertiert alle benannten HTML-Zeichen in ihre entsprechenden Ursprungszeichen
    * htmlentities — Wandelt alle geeigneten Zeichen in entsprechende HTML-Codes um
    * htmlspecialchars_decode — Konvertiert besondere HTML-Auszeichnungen zurück in Buchstaben
    * htmlspecialchars — Wandelt Sonderzeichen in HTML-Codes um
    * strcoll — Locale-basierter Zeichenkettenvergleich
    * strcspn — Ermittelt die Anzahl der nicht übereinstimmenden Zeichen
    */
}