<?php
/**
 * Console Library
 *
 * LICENSE
 *
 * "THE BEER-WARE LICENSE" (Revision 42):
 * "Sven Strittmatter" <ausserirdisch@sven-space.de> wrote this file.
 * As long as you retain this notice you can do whatever you want with
 * this stuff. If we meet some day, and you think this stuff is worth it,
 * you can buy me a beer in return.
 *
 * $Author$
 * $Revisio$
 *
 * @category    Console
 * @package     Unittests
 * @copyright   Copyright (c) 2009 Sven Strittmatter
 */

// Call Console_ApplicationTest::main() if this source file is executed directly.
if (!defined('PHPUnit_MAIN_METHOD')) {
    define('PHPUnit_MAIN_METHOD', 'Console_ApplicationTest::main');
}

/**
 * Test helper
 */
require_once(dirname(__FILE__) . '/../TestHelper.php');

/**
 * Console_Loader
 */
require_once('Console/Application.php');
require_once('Console/Application/Abstract.php');

class TestApplication extends Console_Application_Abstract {
    public static $didSomething = false;

    protected function run() {
        self::$didSomething = true;
    }
}

class BadTestApplication {}

/**
 * @category   Console
 * @package    Unittests
 * @copyright  Copyright (c) 2009 Sven Strittmatter
 */
class Console_ApplicationTest extends PHPUnit_Framework_TestCase {
    /**
     * Runs the test methods of this class.
     *
     * @return void
     */
    public static function main() {
        require_once('PHPUnit/TextUI/TestRunner.php');
        $suite  = new PHPUnit_Framework_TestSuite("Console_ApplicationTest");
        $result = PHPUnit_TextUI_TestRunner::run($suite);
    }

    public function tearDown() {
        TestApplication::$didSomething = false;
    }

    /**
     *
     */
    public function testRunApplication() {
        $this->assertFalse(TestApplication::$didSomething);
        Console_Application::execute('Test');
        $this->assertTrue(TestApplication::$didSomething);
    }

    public function testReturnsExecutedApplication() {
        $application = Console_Application::execute('Test');
        $this->assertTrue($application->isExecuted());
    }

    public function testExecutesApplicationFromDirecotry() {
        $application = Console_Application::execute('HelloWorld', TESTS_CONSOLE_APPLICATION_LOCATION);
        $this->assertTrue($application->isExecuted());
    }

    public function testThrowsExceptionOnFileIsNotReadable() {
        $this->setExpectedException('Console_Exception');
        Console_Application::execute('DoesNotExists');
    }

    public function testThrowsExceptionOnLoadedObjectIsNotInstanceOfConsole_Application_Interface() {
        $this->setExpectedException('Console_Exception');
        Console_Application::execute('BadTest');
    }

    public function testSetsPhpEnviroment() {
        $this->markTestIncomplete();
    }
}

// Call Console_ApplicationTest::main() if this source file is executed directly.
if (PHPUnit_MAIN_METHOD === 'Console_ApplicationTest::main') {
    Console_ApplicationTest::main();
}
