<?php
/**
 * Console Library
 *
 * LICENSE
 *
 * "THE BEER-WARE LICENSE" (Revision 42):
 * "Sven Strittmatter" <ausserirdisch@sven-space.de> wrote this file.
 * As long as you retain this notice you can do whatever you want with
 * this stuff. If we meet some day, and you think this stuff is worth it,
 * you can buy me a beer in return.
 *
 * $Author$
 * $Revisio$
 *
 * @category    Console
 * @package     Unittests
 * @copyright   Copyright (c) 2009 Sven Strittmatter
 */
 
/*
 * Start output buffering
 */
ob_start();

/*
 * Include PHPUnit dependencies
 */
require_once 'PHPUnit/Framework.php';
require_once 'PHPUnit/Framework/IncompleteTestError.php';
require_once 'PHPUnit/Framework/TestCase.php';
require_once 'PHPUnit/Framework/TestSuite.php';
require_once 'PHPUnit/Runner/Version.php';
require_once 'PHPUnit/TextUI/TestRunner.php';
require_once 'PHPUnit/Util/Filter.php';

/*
 * Set error reporting to the level to which Console Library code must comply.
 *
 * PHPUnit 3.3.17 throws e_strict error
 * Strict standards: date(): It is not safe to rely on the system's timezone
 * settings. Please use the date.timezone setting, the TZ environment variable
 * or the date_default_timezone_set() function. In case you used any of those
 * methods and you are still getting this warning, you most likely misspelled
 * the timezone identifier. We selected 'Europe/Berlin' for 'CEST/2.0/DST'
 * instead in /usr/share/php/PHPUnit/Util/Skeleton/Test.php on line 334
 */
error_reporting( E_ALL | ~E_STRICT );

/*
 * Determine the root, library, and tests directories of the framework
 * distribution.
 */
$consoleRoot        = dirname(__FILE__) . DIRECTORY_SEPARATOR . '..';
$consoleCoreLibrary = $consoleRoot . DIRECTORY_SEPARATOR . 'library';
$consoleCoreTests   = $consoleRoot . DIRECTORY_SEPARATOR . 'tests';

/*
 * Omit from code coverage reports the contents of the tests directory
 */
foreach (array('php', 'phtml', 'svn') as $suffix) {
    PHPUnit_Util_Filter::addDirectoryToFilter($consoleCoreTests, ".$suffix");
}

/*
 * Prepend the Console library/ and tests/ directories to the
 * include_path. This allows the tests to run out of the box and helps prevent
 * loading other copies of the framework code and tests that would supersede
 * this copy.
 */
$includePath = array(
    $consoleCoreLibrary,
    $consoleCoreTests,
    get_include_path()
    );
set_include_path(implode(PATH_SEPARATOR, $includePath));

/*
 * Load the user-defined test configuration file, if it exists; otherwise, load
 * the default configuration.
 */
if (is_readable($consoleCoreTests . DIRECTORY_SEPARATOR . 'TestConfiguration.php')) {
    require_once $consoleCoreTests . DIRECTORY_SEPARATOR . 'TestConfiguration.php';
} else {
    $msg  = 'Cant read test config file: ';
    $msg .= $consoleCoreTests . DIRECTORY_SEPARATOR . 'TestConfiguration.php';
    throw new Exception($msg);
}

/*
 * Add Console library/ directory to the PHPUnit code coverage
 * whitelist. This has the effect that only production code source files appear
 * in the code coverage report and that all production code source files, even
 * those that are not covered by a test yet, are processed.
 */
if (defined('TESTS_GENERATE_REPORT') && TESTS_GENERATE_REPORT === true &&
    version_compare(PHPUnit_Runner_Version::id(), '3.1.6', '>=')) {
    PHPUnit_Util_Filter::addDirectoryToWhitelist($consoleCoreLibrary);
}

/*
 * Unset global variables that are no longer needed.
 */
unset($consoleRoot, $consoleCoreLibrary, $consoleCoreTests, $includePath);
